@@ -2,7 +2,7 @@
     smb4kbookmarkhandler_p  -  Private classes for the bookmark handler
                              -------------------
     begin                : Sun Mar 20 2011
-    copyright            : (C) 2011 by Alexander Reinholdt
+    copyright            : (C) 2011-2013 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -23,51 +23,61 @@
  *   MA 02110-1335, USA                                                    *
  ***************************************************************************/
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+// application specific includes
+#include "smb4kbookmarkhandler_p.h"
+#include "smb4ksettings.h"
+#include "smb4kbookmark.h"
+
 // Qt includes
-#include <QVBoxLayout>
-#include <QHBoxLayout>
-#include <QGridLayout>
-#include <QLabel>
-#include <QHeaderView>
-#include <QInputDialog>
-#include <QEvent>
-#include <QDropEvent>
-#include <QDragMoveEvent>
-#include <QDragEnterEvent>
-#include <QDragLeaveEvent>
+#include <QtCore/QEvent>
+#include <QtCore/QTimer>
+#include <QtGui/QVBoxLayout>
+#include <QtGui/QHBoxLayout>
+#include <QtGui/QGridLayout>
+#include <QtGui/QLabel>
+#include <QtGui/QHeaderView>
+#include <QtGui/QDropEvent>
+#include <QtGui/QDragMoveEvent>
+#include <QtGui/QDragEnterEvent>
+#include <QtGui/QDragLeaveEvent>
+#include <QtGui/QTreeWidgetItemIterator>
 
 // KDE includes
 #include <klocale.h>
 #include <kmenu.h>
-
-// application specific includes
-#include <smb4kbookmarkhandler_p.h>
-#include <smb4ksettings.h>
-#include <smb4kbookmark.h>
+#include <kinputdialog.h>
 
 
 Smb4KBookmarkDialog::Smb4KBookmarkDialog( const QList<Smb4KBookmark *> &bookmarks, const QStringList &groups, QWidget *parent )
-: KDialog( parent ), m_bookmarks( bookmarks ), m_groups( groups )
+: KDialog( parent )
 {
   setCaption( i18n( "Add Bookmarks" ) );
   setButtons( Ok|Cancel );
   setDefaultButton( Ok );
 
   setupView();
-  loadLists();
+  loadLists( bookmarks, groups );
 
   KConfigGroup group( Smb4KSettings::self()->config(), "BookmarkDialog" );
   restoreDialogSize( group );
   m_label_edit->completionObject()->setItems( group.readEntry( "LabelCompletion", QStringList() ) );
   m_group_combo->completionObject()->setItems( group.readEntry( "GroupCompletion", m_groups ) );
 
-  connect( this, SIGNAL( buttonClicked( KDialog::ButtonCode ) ), SLOT( slotUserClickedButton( KDialog::ButtonCode ) ) );
-  connect( KGlobalSettings::self(), SIGNAL( iconChanged( int ) ), SLOT( slotIconSizeChanged( int ) ) );
+  connect( this, SIGNAL(buttonClicked(KDialog::ButtonCode)), SLOT(slotUserClickedButton(KDialog::ButtonCode)) );
+  connect( KGlobalSettings::self(), SIGNAL(iconChanged(int)), SLOT(slotIconSizeChanged(int)) );
 }
 
 
 Smb4KBookmarkDialog::~Smb4KBookmarkDialog()
 {
+  while ( !m_bookmarks.isEmpty() )
+  {
+    delete m_bookmarks.takeFirst();
+  }
 }
 
 
@@ -130,26 +140,31 @@ void Smb4KBookmarkDialog::setupView()
 
   setMinimumWidth( sizeHint().width() > 350 ? sizeHint().width() : 350 );
 
-  connect( m_widget, SIGNAL( itemClicked( QListWidgetItem * ) ), SLOT( slotBookmarkClicked( QListWidgetItem * ) ) );
-  connect( m_label_edit, SIGNAL( editingFinished() ), SLOT( slotLabelEdited() ) );
-  connect( m_group_combo->lineEdit(), SIGNAL( editingFinished() ), SLOT( slotGroupEdited() ) );
+  connect( m_widget, SIGNAL(itemClicked(QListWidgetItem*)), SLOT(slotBookmarkClicked(QListWidgetItem*)) );
+  connect( m_label_edit, SIGNAL(editingFinished()), SLOT(slotLabelEdited()) );
+  connect( m_group_combo->lineEdit(), SIGNAL(editingFinished()), SLOT(slotGroupEdited()) );
 }
 
 
-void Smb4KBookmarkDialog::loadLists()
+void Smb4KBookmarkDialog::loadLists( const QList<Smb4KBookmark *> &bookmarks, const QStringList &groups )
 {
-  // Add the bookmarks to the list widget
-  for ( int i = 0; i < m_bookmarks.size(); ++i )
+  // Copy the bookmarks to the internal list and add them to 
+  // the list widget afterwards.
+  for ( int i = 0; i < bookmarks.size(); ++i )
   {
-    QListWidgetItem *item = new QListWidgetItem( m_bookmarks.at( i )->icon(), m_bookmarks.at( i )->unc(), m_widget );
-    item->setData( Qt::UserRole, m_bookmarks.at( i )->url() );
+    Smb4KBookmark *bookmark = new Smb4KBookmark( *bookmarks[i] );
+    QListWidgetItem *item = new QListWidgetItem( bookmark->icon(), bookmark->unc(), m_widget );
+    item->setData( Qt::UserRole, static_cast<QUrl>( bookmark->url() ) );
+    
+    m_bookmarks << bookmark;
   }
 
+  m_groups = groups;
   m_group_combo->addItems( m_groups );
 }
 
 
-Smb4KBookmark *Smb4KBookmarkDialog::findBookmark( const QUrl &url )
+Smb4KBookmark *Smb4KBookmarkDialog::findBookmark( const KUrl &url )
 {
   Smb4KBookmark *bookmark = NULL;
   
@@ -184,14 +199,14 @@ void Smb4KBookmarkDialog::slotBookmarkClicked( QListWidgetItem *bookmark_item )
       // Do nothing
     }
 
-    QUrl url = bookmark_item->data( Qt::UserRole ).toUrl();
+    KUrl url = bookmark_item->data( Qt::UserRole ).toUrl();
     
     Smb4KBookmark *bookmark = findBookmark( url );
 
     if ( bookmark )
     {
       m_label_edit->setText( bookmark->label() );
-      m_group_combo->setCurrentItem( bookmark->group() );
+      m_group_combo->setCurrentItem( bookmark->groupName() );
     }
     else
     {
@@ -212,7 +227,7 @@ void Smb4KBookmarkDialog::slotBookmarkClicked( QListWidgetItem *bookmark_item )
 void Smb4KBookmarkDialog::slotLabelEdited()
 {
   // Set the label
-  QUrl url = m_widget->currentItem()->data( Qt::UserRole ).toUrl();
+  KUrl url = m_widget->currentItem()->data( Qt::UserRole ).toUrl();
 
   Smb4KBookmark *bookmark = findBookmark( url );
 
@@ -242,13 +257,13 @@ void Smb4KBookmarkDialog::slotLabelEdited()
 void Smb4KBookmarkDialog::slotGroupEdited()
 {
   // Set the group
-  QUrl url = m_widget->currentItem()->data( Qt::UserRole ).toUrl();
+  KUrl url = m_widget->currentItem()->data( Qt::UserRole ).toUrl();
 
   Smb4KBookmark *bookmark = findBookmark( url );
 
   if ( bookmark )
   {
-    bookmark->setGroup( m_group_combo->currentText() );
+    bookmark->setGroupName( m_group_combo->currentText() );
   }
   else
   {
@@ -337,26 +352,23 @@ Smb4KBookmarkEditor::Smb4KBookmarkEditor( const QList<Smb4KBookmark *> &bookmark
   m_ip_edit->completionObject()->setItems( group.readEntry( "IPCompletion", QStringList() ) );
   m_group_combo->completionObject()->setItems( group.readEntry( "GroupCompletion", m_groups ) );
 
-  connect( this, SIGNAL( buttonClicked( KDialog::ButtonCode ) ), SLOT( slotUserClickedButton( KDialog::ButtonCode ) ) );
-  connect( KGlobalSettings::self(), SIGNAL( iconChanged( int ) ), SLOT( slotIconSizeChanged( int ) ) );
+  connect( this, SIGNAL(buttonClicked(KDialog::ButtonCode)), SLOT(slotUserClickedButton(KDialog::ButtonCode)) );
+  connect( KGlobalSettings::self(), SIGNAL(iconChanged(int)), SLOT(slotIconSizeChanged(int)) );
 }
 
 
 Smb4KBookmarkEditor::~Smb4KBookmarkEditor()
 {
+  while ( !m_bookmarks.isEmpty() )
+  {
+    delete m_bookmarks.takeFirst();
+  }
 }
 
 
-QList<Smb4KBookmark *> Smb4KBookmarkEditor::editedBookmarks()
+QList<Smb4KBookmark *> Smb4KBookmarkEditor::editedBookmarks() const
 {
-  QList<Smb4KBookmark *> list;
-
-  for ( int i = 0; i < m_bookmarks.size(); ++i )
-  {
-    list << &m_bookmarks[i];
-  }
-
-  return list;
+  return m_bookmarks;
 }
 
 
@@ -368,90 +380,26 @@ bool Smb4KBookmarkEditor::eventFilter( QObject *obj, QEvent *e )
     {
       case QEvent::DragEnter:
       {
-        // Only accept the drag enter event if it originates from
-        // the tree widget's viewport.
-        // NOTE: Do not remove the drag item from the parent (neither
-        // widget nor top level item). You will crash the application
-        // otherwise.
         QDragEnterEvent *ev = static_cast<QDragEnterEvent *>( e );
-        QPoint pos = m_tree_widget->viewport()->mapToParent( ev->pos() );
-
-        if ( (m_drag_item = m_tree_widget->itemAt( pos )) == NULL )
+        
+        if ( ev->source() == m_tree_widget->viewport() )
         {
-          ev->ignore();
+          e->accept();
         }
         else
         {
-          ev->accept();
+          e->ignore();
         }
         break;
       }
       case QEvent::DragLeave:
       {
-        // Do not allow a bookmark to be dragged outside the tree widget.
-        QDragLeaveEvent *ev = static_cast<QDragLeaveEvent *>( e );
-        ev->ignore();
+        e->ignore();
         break;
       }
       case QEvent::Drop:
       {
-        // Get the bookmark for the dropped item.
-        Smb4KBookmark *bookmark = findBookmark( m_drag_item->data( 0, QTreeWidgetItem::UserType ).toUrl() );
-        
-        // Get the item where the dragged item has been dropped.
-        QDropEvent *ev = static_cast<QDropEvent *>( e );
-        QPoint pos = m_tree_widget->viewport()->mapToParent( ev->pos() );
-        QTreeWidgetItem *neighbor = m_tree_widget->itemAt( pos );
-        
-        if ( neighbor )
-        {
-          QTreeWidgetItem *parent = NULL;
-          QString group_name;
-          
-          if ( m_tree_widget->indexOfTopLevelItem( neighbor ) != -1 )
-          {
-            // This is the group item.
-            parent = neighbor;
-            neighbor->addChild( m_drag_item );
-            group_name = neighbor->text( 0 );
-          }
-          else
-          {
-            // This is a child.
-            parent = neighbor->parent();
-            neighbor->parent()->addChild( m_drag_item );
-            group_name = neighbor->parent()->text( 0 );
-          }
-
-          if ( bookmark )
-          {
-            bookmark->setGroup( group_name );
-          }
-          else
-          {
-            // Do nothing
-          }
-
-          parent->sortChildren( (m_tree_widget->columnCount() - 1), Qt::AscendingOrder );
-        }
-        else
-        {
-          // The item has been dragged to the top level. Add it as top level
-          // item.
-          m_tree_widget->addTopLevelItem( m_drag_item );
-
-          if ( bookmark )
-          {
-            bookmark->setGroup( QString() );
-          }
-          else
-          {
-            // Do nothing
-          }
-
-          m_tree_widget->sortItems( (m_tree_widget->columnCount() - 1), Qt::AscendingOrder );
-        }
-        m_drag_item = NULL;
+        QTimer::singleShot( 50, this, SLOT(slotAdjust()));
         break;
       }
       default:
@@ -531,81 +479,75 @@ void Smb4KBookmarkEditor::setupView()
   layout->addWidget( m_tree_widget );
   layout->addWidget( m_editors );
 
-  connect( m_tree_widget, SIGNAL( itemClicked( QTreeWidgetItem *, int ) ), SLOT( slotItemClicked( QTreeWidgetItem *, int ) ) );
-  connect( m_tree_widget, SIGNAL( customContextMenuRequested( const QPoint & ) ), SLOT( slotContextMenuRequested( const QPoint & ) ) );
-  connect( m_label_edit, SIGNAL( editingFinished() ), SLOT( slotLabelEdited() ) );
-  connect( m_ip_edit, SIGNAL( editingFinished() ), SLOT( slotIPEdited() ) );
-  connect( m_login_edit, SIGNAL( editingFinished() ), SLOT( slotLoginEdited() ) );
-  connect( m_group_combo->lineEdit(), SIGNAL( editingFinished() ), SLOT( slotGroupEdited() ) );
-  connect( m_add_group, SIGNAL( triggered( bool ) ), SLOT( slotAddGroupTriggered( bool ) ) );
-  connect( m_delete, SIGNAL( triggered( bool ) ), SLOT( slotDeleteTriggered( bool ) ) );
-  connect( m_clear, SIGNAL( triggered( bool ) ), SLOT( slotClearTriggered( bool ) ) );
+  connect( m_tree_widget, SIGNAL(itemClicked(QTreeWidgetItem*,int)), SLOT(slotItemClicked(QTreeWidgetItem*,int)) );
+  connect( m_tree_widget, SIGNAL(customContextMenuRequested(QPoint)), SLOT(slotContextMenuRequested(QPoint)) );
+  connect( m_label_edit, SIGNAL(editingFinished()), SLOT(slotLabelEdited()) );
+  connect( m_ip_edit, SIGNAL(editingFinished()), SLOT(slotIPEdited()) );
+  connect( m_login_edit, SIGNAL(editingFinished()), SLOT(slotLoginEdited()) );
+  connect( m_group_combo->lineEdit(), SIGNAL(editingFinished()), SLOT(slotGroupEdited()) );
+  connect( m_add_group, SIGNAL(triggered(bool)), SLOT(slotAddGroupTriggered(bool)) );
+  connect( m_delete, SIGNAL(triggered(bool)), SLOT(slotDeleteTriggered(bool)) );
+  connect( m_clear, SIGNAL(triggered(bool)), SLOT(slotClearTriggered(bool)) );
 }
 
 
-void Smb4KBookmarkEditor::loadBookmarks( const QList<Smb4KBookmark *> &list )
+void Smb4KBookmarkEditor::loadBookmarks( const QList<Smb4KBookmark *> &bookmarks )
 {
-  for ( int i = 0; i < list.size(); ++i )
+  // Copy the bookmarks and the groups to the internal lists.
+  for ( int i = 0; i < bookmarks.size(); ++i )
   {
-    m_bookmarks << *list[i];
-  }
-  
-  // Load the bookmarks into the tree view and compile 
-  // the available groups in a list.
-  for ( int i = 0; i < m_bookmarks.size(); ++i )
-  {
-    if ( !m_bookmarks.at( i ).group().isEmpty() )
+    m_bookmarks << new Smb4KBookmark( *bookmarks[i] );
+    
+    if ( !m_groups.contains( bookmarks.at( i )->groupName() ) )
     {
-      // Find the group and add the bookmark
-      QList<QTreeWidgetItem *> items = m_tree_widget->findItems( m_bookmarks.at( i ).group(), Qt::MatchFixedString|Qt::MatchCaseSensitive, 0 );
-      QTreeWidgetItem *group = NULL;
-
-      if ( !items.isEmpty() )
-      {
-        group = items.first();
-      }
-      else
-      {
-        group = new QTreeWidgetItem( QTreeWidgetItem::UserType );
-        group->setIcon( 0, KIcon( "folder-favorites" ) );
-        group->setText( 0, m_bookmarks.at( i ).group() );
-        group->setText( (m_tree_widget->columnCount() - 1), QString( "00_%1" ).arg( m_bookmarks.at( i ).group() ) );
-        group->setFlags( Qt::ItemIsSelectable|Qt::ItemIsUserCheckable|Qt::ItemIsEnabled|Qt::ItemIsDropEnabled ) ;
-        m_tree_widget->addTopLevelItem( group );
-      }
-
-      QTreeWidgetItem *bookmark = new QTreeWidgetItem( QTreeWidgetItem::UserType );
-      bookmark->setData( 0, QTreeWidgetItem::UserType, m_bookmarks.at( i ).url() );
-      bookmark->setIcon( 0, m_bookmarks.at( i ).icon() );
-      bookmark->setText( 0, m_bookmarks.at( i ).unc() );
-      bookmark->setText( (m_tree_widget->columnCount() - 1), QString( "01_%1" ).arg( m_bookmarks.at( i ).unc() ) );
-      bookmark->setFlags( Qt::ItemIsSelectable|Qt::ItemIsUserCheckable|Qt::ItemIsEnabled|Qt::ItemIsDragEnabled ) ;
-      group->addChild( bookmark );
-
-      group->setExpanded( true );
-    }
-    else
-    {
-      QTreeWidgetItem *bookmark = new QTreeWidgetItem( QTreeWidgetItem::UserType );
-      bookmark->setData( 0, QTreeWidgetItem::UserType, m_bookmarks.at( i ).url() );
-      bookmark->setIcon( 0, m_bookmarks.at( i ).icon() );
-      bookmark->setText( 0, m_bookmarks.at( i ).unc() );
-      bookmark->setText( (m_tree_widget->columnCount() - 1), QString( "01_%1" ).arg( m_bookmarks.at( i ).unc() ) );
-      bookmark->setFlags( Qt::ItemIsSelectable|Qt::ItemIsUserCheckable|Qt::ItemIsEnabled|Qt::ItemIsDragEnabled ) ;
-      m_tree_widget->addTopLevelItem( bookmark );
-    }
-
-    // Add the group to the list
-    if ( !m_groups.contains( m_bookmarks.at( i ).group() ) )
-    {
-      m_groups << m_bookmarks.at( i ).group();
+      m_groups << bookmarks.at( i )->groupName();
     }
     else
     {
       // Do nothing
     }
   }
-
+  
+  // Insert the groups into the tree widget.
+  for ( int i = 0; i < m_groups.size(); ++i )
+  {
+    if ( !m_groups.at( i ).isEmpty() )
+    {
+      QTreeWidgetItem *group = new QTreeWidgetItem( QTreeWidgetItem::UserType );
+      group->setIcon( 0, KIcon( "folder-favorites" ) );
+      group->setText( 0, m_bookmarks.at( i )->groupName() );
+      group->setText( (m_tree_widget->columnCount() - 1), QString( "00_%1" ).arg( m_bookmarks.at( i )->groupName() ) );
+      group->setFlags( Qt::ItemIsSelectable|Qt::ItemIsUserCheckable|Qt::ItemIsEnabled|Qt::ItemIsDropEnabled ) ;
+      m_tree_widget->addTopLevelItem( group );
+    }
+    else
+    {
+      // Do nothing
+    }
+  }
+  
+  // Now load the bookmarks
+  for ( int i = 0; i < m_bookmarks.size(); ++i )
+  {
+    QTreeWidgetItem *bookmark = new QTreeWidgetItem( QTreeWidgetItem::UserType );
+    bookmark->setData( 0, QTreeWidgetItem::UserType, static_cast<QUrl>( m_bookmarks.at( i )->url() ) );
+    bookmark->setIcon( 0, m_bookmarks.at( i )->icon() );
+    bookmark->setText( 0, m_bookmarks.at( i )->unc() );
+    bookmark->setText( (m_tree_widget->columnCount() - 1), QString( "01_%1" ).arg( m_bookmarks.at( i )->unc() ) );
+    bookmark->setFlags( Qt::ItemIsSelectable|Qt::ItemIsUserCheckable|Qt::ItemIsEnabled|Qt::ItemIsDragEnabled ) ;
+    
+    if ( !m_bookmarks.at( i )->groupName().isEmpty() )
+    {
+      QList<QTreeWidgetItem *> items = m_tree_widget->findItems( m_bookmarks.at( i )->groupName(), Qt::MatchFixedString|Qt::MatchCaseSensitive, 0 );
+      items.first()->addChild( bookmark );
+      items.first()->setExpanded( true );      
+    }
+    else
+    {
+      m_tree_widget->addTopLevelItem( bookmark );
+    }
+  }
+  
   // Sort
   for ( int i = 0; i < m_tree_widget->topLevelItemCount(); ++i )
   {
@@ -613,7 +555,7 @@ void Smb4KBookmarkEditor::loadBookmarks( const QList<Smb4KBookmark *> &list )
   }
   
   m_tree_widget->sortItems( (m_tree_widget->columnCount() - 1), Qt::AscendingOrder );
-
+  
   // Check that an empty goup entry is also present. If it is not there,
   // add it now and insert the groups to the group combo box afterwards.
   if ( !m_groups.contains( "" ) && !m_groups.contains( QString() ) )
@@ -630,15 +572,15 @@ void Smb4KBookmarkEditor::loadBookmarks( const QList<Smb4KBookmark *> &list )
 }
 
 
-Smb4KBookmark *Smb4KBookmarkEditor::findBookmark( const QUrl &url )
+Smb4KBookmark *Smb4KBookmarkEditor::findBookmark( const KUrl &url )
 {
   Smb4KBookmark *bookmark = NULL;
 
   for ( int i = 0; i < m_bookmarks.size(); ++i )
   {
-    if ( m_bookmarks.at( i ).url() == url )
+    if ( m_bookmarks.at( i )->url() == url )
     {
-      bookmark = &m_bookmarks[i];
+      bookmark = m_bookmarks[i];
       break;
     }
     else
@@ -669,7 +611,7 @@ void Smb4KBookmarkEditor::slotItemClicked( QTreeWidgetItem *item, int /*col*/ )
           m_label_edit->setText( bookmark->label() );
           m_login_edit->setText( bookmark->login() );
           m_ip_edit->setText( bookmark->hostIP() );
-          m_group_combo->setCurrentItem( bookmark->group() );
+          m_group_combo->setCurrentItem( bookmark->groupName() );
           m_editors->setEnabled( true );
         }
         else
@@ -700,7 +642,7 @@ void Smb4KBookmarkEditor::slotItemClicked( QTreeWidgetItem *item, int /*col*/ )
         m_label_edit->setText( bookmark->label() );
         m_login_edit->setText( bookmark->login() );
         m_ip_edit->setText( bookmark->hostIP() );
-        m_group_combo->setCurrentItem( bookmark->group() );
+        m_group_combo->setCurrentItem( bookmark->groupName() );
         m_editors->setEnabled( true );
       }
       else
@@ -735,7 +677,7 @@ void Smb4KBookmarkEditor::slotContextMenuRequested( const QPoint &pos )
 void Smb4KBookmarkEditor::slotLabelEdited()
 {
   // Set the label
-  QUrl url = m_tree_widget->currentItem()->data( 0, QTreeWidgetItem::UserType ).toUrl();
+  KUrl url = m_tree_widget->currentItem()->data( 0, QTreeWidgetItem::UserType ).toUrl();
 
   Smb4KBookmark *bookmark = findBookmark( url );
 
@@ -765,7 +707,7 @@ void Smb4KBookmarkEditor::slotLabelEdited()
 void Smb4KBookmarkEditor::slotLoginEdited()
 {
   // Set the login
-  QUrl url = m_tree_widget->currentItem()->data( 0, QTreeWidgetItem::UserType ).toUrl();
+  KUrl url = m_tree_widget->currentItem()->data( 0, QTreeWidgetItem::UserType ).toUrl();
 
   Smb4KBookmark *bookmark = findBookmark( url );
 
@@ -795,7 +737,7 @@ void Smb4KBookmarkEditor::slotLoginEdited()
 void Smb4KBookmarkEditor::slotIPEdited()
 {
   // Set the ip address
-  QUrl url = m_tree_widget->currentItem()->data( 0, QTreeWidgetItem::UserType ).toUrl();
+  KUrl url = m_tree_widget->currentItem()->data( 0, QTreeWidgetItem::UserType ).toUrl();
 
   Smb4KBookmark *bookmark = findBookmark( url );
 
@@ -838,7 +780,7 @@ void Smb4KBookmarkEditor::slotGroupEdited()
   // Get the URL of the current item. We need to do this
   // here, because after the following operation there is
   // no current item anymore.
-  QUrl url = m_tree_widget->currentItem()->data( 0, QTreeWidgetItem::UserType ).toUrl();
+  KUrl url = m_tree_widget->currentItem()->data( 0, QTreeWidgetItem::UserType ).toUrl();
 
   // Return here if the item is a group
   if ( url.isEmpty() )
@@ -854,7 +796,7 @@ void Smb4KBookmarkEditor::slotGroupEdited()
 
   if ( bookmark )
   {
-    bookmark->setGroup( m_group_combo->currentText() );
+    bookmark->setGroupName( m_group_combo->currentText() );
   }
   else
   {
@@ -1018,8 +960,8 @@ void Smb4KBookmarkEditor::slotAddGroupTriggered( bool /*checked*/ )
 {
   bool ok = false;
   
-  QString group_name = QInputDialog::getText( this, i18n( "Add Group" ), i18n( "Group name:" ),
-                                              QLineEdit::Normal, QString(), &ok );
+  QString group_name = KInputDialog::getText( i18n( "Add Group" ), i18n( "Group name:" ),
+                                              QString(), &ok, this );
 
   if ( ok && !group_name.isEmpty() &&
        m_tree_widget->findItems( group_name, Qt::MatchFixedString|Qt::MatchCaseSensitive, 0 ).isEmpty() )
@@ -1072,13 +1014,13 @@ void Smb4KBookmarkEditor::slotUserClickedButton( KDialog::ButtonCode code )
       // Remove obsolete bookmarks.
       // We can assume that each server in the network has a unique 
       // name, so we only need to test for the UNC and are done.
-      QMutableListIterator<Smb4KBookmark> it( m_bookmarks );
+      QMutableListIterator<Smb4KBookmark *> it( m_bookmarks );
 
       while ( it.hasNext() )
       {
-        Smb4KBookmark bookmark = it.next();
+        Smb4KBookmark *bookmark = it.next();
         
-        QList<QTreeWidgetItem *> items = m_tree_widget->findItems( bookmark.unc(), Qt::MatchFixedString|Qt::MatchCaseSensitive|Qt::MatchRecursive, 0 );
+        QList<QTreeWidgetItem *> items = m_tree_widget->findItems( bookmark->unc(), Qt::MatchFixedString|Qt::MatchCaseSensitive|Qt::MatchRecursive, 0 );
 
         if ( items.isEmpty() )
         {
@@ -1124,15 +1066,57 @@ void Smb4KBookmarkEditor::slotIconSizeChanged( int group )
 }
 
 
-
-Smb4KBookmarkHandlerPrivate::Smb4KBookmarkHandlerPrivate()
+void Smb4KBookmarkEditor::slotAdjust()
 {
+  // Do the necessary adjustments:
+  QTreeWidgetItemIterator it( m_tree_widget );
+  while ( *it )
+  {
+    if ( !(*it)->parent() )
+    {
+      if ( (*it)->data( 0, QTreeWidgetItem::UserType ).toUrl().isEmpty() )
+      {
+        if ( (*it)->childCount() == 0 )
+        {
+          delete *it;
+        }
+        else
+        {
+          // Do nothing
+        }
+      }
+      else
+      {
+        Smb4KBookmark *bookmark = findBookmark((*it)->data( 0, QTreeWidgetItem::UserType ).toUrl());
+      
+        if ( bookmark )
+        {
+          bookmark->setGroupName("");
+        }
+        else
+        {
+          // Do nothing
+        }        
+      }
+    }
+    else
+    {
+      Smb4KBookmark *bookmark = findBookmark((*it)->data( 0, QTreeWidgetItem::UserType ).toUrl());
+      
+      if ( bookmark )
+      {
+        bookmark->setGroupName((*it)->parent()->text(0));
+      }
+      else
+      {
+        // Do nothing
+      }
+    }
+    ++it;
+  }
 }
 
 
-Smb4KBookmarkHandlerPrivate::~Smb4KBookmarkHandlerPrivate()
-{
-}
 
 #include "smb4kbookmarkhandler_p.moc"
 
