@@ -3,7 +3,7 @@
     namespace.
                              -------------------
     begin                : Di Jul 24 2007
-    copyright            : (C) 2007-2011 by Alexander Reinholdt
+    copyright            : (C) 2007-2012 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -20,29 +20,38 @@
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
- *   Free Software Foundation, 51 Franklin Street, Suite 500, Boston,      *
+ *   Free Software Foundation, Inc., 51 Franklin Street, Suite 500, Boston,*
  *   MA 02110-1335, USA                                                    *
  ***************************************************************************/
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+// application specific includes
+#include "smb4kglobal_p.h"
+#include "smb4knotification.h"
+#include "smb4ksettings.h"
+
 // Qt includes
-#include <QDir>
-#include <QTextStream>
-#include <QTextCodec>
-#include <QFile>
+#include <QtCore/QDir>
+#include <QtCore/QTextStream>
+#include <QtCore/QTextCodec>
+#include <QtCore/QFile>
+#include <QtCore/QCoreApplication>
+#include <QtNetwork/QHostAddress>
+#include <QtNetwork/QAbstractSocket>
 
 // system specific includes
 #include <unistd.h>
 #include <sys/types.h>
 #include <errno.h>
 
-// application specific includes
-#include "smb4kglobal_p.h"
-#include <smb4knotification.h>
-
 
 Smb4KGlobalPrivate::Smb4KGlobalPrivate()
 {
   onlyForeignShares = false;
+  coreInitialized = false;
 }
 
 
@@ -132,17 +141,17 @@ const QMap<QString,QString> &Smb4KGlobalPrivate::globalSambaOptions( bool read )
       // Process the file contents.
       for ( int i = contents.indexOf( "[global]", 0 ); i < contents.size(); ++i )
       {
-        if ( contents.at( i ).trimmed().startsWith( "#" ) ||
-            contents.at( i ).trimmed().startsWith( ";" ) )
+        if ( contents.at( i ).trimmed().startsWith( '#' ) ||
+             contents.at( i ).trimmed().startsWith( ';' ) )
         {
           // This is a comment. We do not need it.
           continue;
         }
-        else if ( contents.at( i ).trimmed().startsWith( "include" ) )
+        else if ( contents.at( i ).trimmed().startsWith( QLatin1String( "include" ) ) )
         {
           // Look for the include file and put its contents into the
           // m_samba_options map.
-          file.setFileName( contents.at( i ).section( "=", 1, 1 ).trimmed() );
+          file.setFileName( contents.at( i ).section( '=', 1, 1 ).trimmed() );
 
           if ( file.exists() )
           {
@@ -159,14 +168,14 @@ const QMap<QString,QString> &Smb4KGlobalPrivate::globalSambaOptions( bool read )
               {
                 buffer = ts.readLine( 0 ).trimmed();
 
-                if ( buffer.startsWith( "#" ) || buffer.startsWith( ";" ) )
+                if ( buffer.startsWith( '#' ) || buffer.startsWith( ';' ) )
                 {
                   continue;
                 }
                 else
                 {
-                  QString key = buffer.section( "=", 0, 0 ).trimmed().toLower();
-                  m_samba_options[key] = buffer.section( "=", 1, 1 ).trimmed().toUpper();
+                  QString key = buffer.section( '=', 0, 0 ).trimmed().toLower();
+                  m_samba_options[key] = buffer.section( '=', 1, 1 ).trimmed().toUpper();
 
                   continue;
                 }
@@ -180,7 +189,7 @@ const QMap<QString,QString> &Smb4KGlobalPrivate::globalSambaOptions( bool read )
             }
           }
         }
-        else if ( contents.at( i ).startsWith( "[" ) &&
+        else if ( contents.at( i ).startsWith( '[' ) &&
                   !contents.at( i ).contains( "[global]", Qt::CaseSensitive ) )
         {
           // We reached the end of the [global] section. Stop here.
@@ -190,8 +199,8 @@ const QMap<QString,QString> &Smb4KGlobalPrivate::globalSambaOptions( bool read )
         {
           // Put the entries of the [global] section into the m_samba_options
           // map.
-          QString key = contents.at( i ).section( "=", 0, 0 ).trimmed().toLower();
-          m_samba_options[key] = contents.at( i ).section( "=", 1, 1 ).trimmed().toUpper();
+          QString key = contents.at( i ).section( '=', 0, 0 ).trimmed().toLower();
+          m_samba_options[key] = contents.at( i ).section( '=', 1, 1 ).trimmed().toUpper();
         }
       }
     }
@@ -231,3 +240,87 @@ const QMap<QString,QString> &Smb4KGlobalPrivate::globalSambaOptions( bool read )
 }
 
 
+void Smb4KGlobalPrivate::setDefaultSettings()
+{
+  // Samba options that have to be dynamically imported from smb.conf:
+  QMap<QString, QString> opts = globalSambaOptions( true );
+
+  if ( !opts["netbios name"].isEmpty() )
+  {
+    Smb4KSettings::self()->netBIOSNameItem()->setDefaultValue( opts["netbios name"] );
+
+    if ( Smb4KSettings::netBIOSName().isEmpty() )
+    {
+      Smb4KSettings::self()->netBIOSNameItem()->setDefault();
+    }
+  }
+
+  if ( !opts["workgroup"].isEmpty() )
+  {
+    Smb4KSettings::self()->domainNameItem()->setDefaultValue( opts["workgroup"] );
+
+    if ( Smb4KSettings::domainName().isEmpty() )
+    {
+      Smb4KSettings::self()->domainNameItem()->setDefault();
+    }
+  }
+
+  if ( !opts["socket options"].isEmpty() )
+  {
+    Smb4KSettings::self()->socketOptionsItem()->setDefaultValue( opts["socket options"] );
+
+    if ( Smb4KSettings::socketOptions().isEmpty() )
+    {
+      Smb4KSettings::self()->socketOptionsItem()->setDefault();
+    }
+  }
+
+  if ( !opts["netbios scope"].isEmpty() )
+  {
+    Smb4KSettings::self()->netBIOSScopeItem()->setDefaultValue( opts["netbios scope"] );
+
+    if ( Smb4KSettings::netBIOSScope().isEmpty() )
+    {
+      Smb4KSettings::self()->netBIOSScopeItem()->setDefault();
+    }
+  }
+
+  if ( !opts["name resolve order"].isEmpty() )
+  {
+    Smb4KSettings::self()->nameResolveOrderItem()->setDefaultValue( opts["name resolve order"] );
+
+    if ( Smb4KSettings::nameResolveOrder().isEmpty() )
+    {
+      Smb4KSettings::self()->nameResolveOrderItem()->setDefault();
+    }
+  }
+
+  QHostAddress address( opts["interfaces"].section( ' ', 0, 0 ) );
+
+  if ( address.protocol() != QAbstractSocket::UnknownNetworkLayerProtocol )
+  {
+    Smb4KSettings::self()->broadcastAddressItem()->setDefaultValue( address.toString() );
+
+    if ( Smb4KSettings::broadcastAddress().isEmpty() )
+    {
+      Smb4KSettings::self()->broadcastAddressItem()->setDefault();
+    }
+  }
+}
+
+
+void Smb4KGlobalPrivate::makeConnections()
+{
+  connect( QCoreApplication::instance(), SIGNAL(aboutToQuit()), SLOT(slotAboutToQuit()) );
+}
+
+
+void Smb4KGlobalPrivate::slotAboutToQuit()
+{
+  Smb4KSettings::self()->writeConfig();
+}
+
+
+#include "smb4kglobal_p.moc"
+
+
