@@ -3,7 +3,7 @@
     Smb4KGlobal namespace.
                              -------------------
     begin                : Di Jul 24 2007
-    copyright            : (C) 2007-2011 by Alexander Reinholdt
+    copyright            : (C) 2007-2012 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -20,34 +20,33 @@
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
- *   Free Software Foundation, 51 Franklin Street, Suite 500, Boston,      *
+ *   Free Software Foundation, Inc., 51 Franklin Street, Suite 500, Boston,*
  *   MA 02110-1335, USA                                                    *
  ***************************************************************************/
 
 #ifndef SMB4KGLOBAL_P_H
 #define SMB4KGLOBAL_P_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+// application specific includes
+#include "smb4kworkgroup.h"
+#include "smb4khost.h"
+#include "smb4kshare.h"
 
 // Qt includes
-#include <QList>
-#include <QMap>
-
-// application specific includes
-#include <smb4kworkgroup.h>
-#include <smb4khost.h>
-#include <smb4kshare.h>
+#include <QtCore/QList>
+#include <QtCore/QMap>
+#include <QtCore/QObject>
 
 /**
  * This class is a private helper for the Smb4KGlobal namespace.
  *
- * @author Alexander Reinholdt <dustpuppy@users.berlios.de>
+ * @author Alexander Reinholdt <alexander.reinholdt@kdemail.net>
  */
 
-class Smb4KGlobalPrivate
+class Smb4KGlobalPrivate : public QObject
 {
+  Q_OBJECT
+  
   public:
     /**
      * Constructor
@@ -89,7 +88,35 @@ class Smb4KGlobalPrivate
      * are in the list of mounted shares
      */
     bool onlyForeignShares;
+
+    /**
+     * Set default values for some settings
+     */
+    void setDefaultSettings();
+
+    /**
+     * Make connections
+     */
+    void makeConnections();
+
+    /**
+     * Boolean that is TRUE if the core classes have
+     * been initialized
+     */
+    bool coreInitialized;
     
+    /**
+     * Boolean that determines if the core classes should set
+     * a busy cursor when they are doing something or not.
+     */
+    bool modifyCursor;
+
+  protected Q_SLOTS:
+    /**
+     * This slot does last things before the application quits
+     */
+    void slotAboutToQuit();
+
   private:
     QMap<QString,QString> m_samba_options;
 };
