@@ -3,7 +3,7 @@
     Smb4KMounter class.
                              -------------------
     begin                : Do Jul 19 2007
-    copyright            : (C) 2007-2011 by Alexander Reinholdt
+    copyright            : (C) 2007-2012 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -20,18 +20,33 @@
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
- *   Free Software Foundation, 51 Franklin Street, Suite 500, Boston,      *
+ *   Free Software Foundation, Inc., 51 Franklin Street, Suite 500, Boston,*
  *   MA 02110-1335, USA                                                    *
  ***************************************************************************/
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+// application specific includes
+#include "smb4kmounter_p.h"
+#include "smb4ksettings.h"
+#include "smb4knotification.h"
+#include "smb4khomesshareshandler.h"
+#include "smb4kglobal.h"
+#include "smb4kcustomoptionsmanager.h"
+#include "smb4kcustomoptions.h"
+
 // Qt includes
-#include <QFileInfo>
-#include <QHBoxLayout>
-#include <QVBoxLayout>
-#include <QGridLayout>
-#include <QLabel>
-#include <QTimer>
-#include <QDir>
+#include <QtCore/QFileInfo>
+#include <QtCore/QFile>
+#include <QtCore/QTimer>
+#include <QtCore/QDir>
+#include <QtCore/QProcessEnvironment>
+#include <QtGui/QHBoxLayout>
+#include <QtGui/QVBoxLayout>
+#include <QtGui/QGridLayout>
+#include <QtGui/QLabel>
 
 // KDE includes
 #include <kdiskfreespaceinfo.h>
@@ -40,14 +55,10 @@
 #include <kmountpoint.h>
 #include <kshell.h>
 
-// application specific includes
-#include <smb4kmounter_p.h>
-#include <smb4ksettings.h>
-#include <smb4knotification.h>
-#include <smb4khomesshareshandler.h>
-#include <smb4kglobal.h>
-#include <smb4kcustomoptionsmanager.h>
-#include <smb4kcustomoptions.h>
+// system includes
+#include <unistd.h>
+#include <sys/types.h>
+       
 
 using namespace Smb4KGlobal;
 
@@ -61,20 +72,34 @@ Smb4KMountJob::Smb4KMountJob( QObject *parent ) : KJob( parent ),
 
 Smb4KMountJob::~Smb4KMountJob()
 {
+  while ( !m_shares.isEmpty() )
+  {
+    delete m_shares.takeFirst();
+  }
+
+  while ( !m_auth_errors.isEmpty() )
+  {
+    delete m_auth_errors.takeFirst();
+  }
+
+  while ( !m_retries.isEmpty() )
+  {
+    delete m_retries.takeFirst();
+  }
 }
 
 
 void Smb4KMountJob::start()
 {
   m_started = true;
-  QTimer::singleShot( 0, this, SLOT( slotStartMount() ) );
+  QTimer::singleShot( 50, this, SLOT(slotStartMount()) );
 }
 
 
 void Smb4KMountJob::setupMount( Smb4KShare *share, QWidget *parent )
 {
   Q_ASSERT( share );
-  m_shares << *share;
+  m_shares << new Smb4KShare( *share );
   m_parent_widget = parent;
 }
 
@@ -87,7 +112,7 @@ void Smb4KMountJob::setupMount( const QList<Smb4KShare*> &shares, QWidget *paren
   {
     Smb4KShare *share = it.next();
     Q_ASSERT( share );
-    m_shares << *share;
+    m_shares << new Smb4KShare( *share );
   }
 
   m_parent_widget = parent;
@@ -454,54 +479,127 @@ bool Smb4KMountJob::createMountAction( Smb4KShare *share, Action *action )
   }
 
   // Security mode
-  switch ( Smb4KSettings::securityMode() )
+  if ( options )
   {
-    case Smb4KSettings::EnumSecurityMode::None:
+    switch ( options->securityMode() )
     {
-      args_list << "sec=none";
-      break;
+      case Smb4KCustomOptions::NoSecurityMode:
+      {
+        args_list << "sec=none";
+        break;
+      }
+      case Smb4KCustomOptions::Krb5:
+      {
+        args_list << "sec=krb5";
+        args_list << QString( "cruid=%1" ).arg( getuid() );
+        break;
+      }
+      case Smb4KCustomOptions::Krb5i:
+      {
+        args_list << "sec=krb5i";
+        args_list << QString( "cruid=%1" ).arg( getuid() );
+        break;
+      }
+      case Smb4KCustomOptions::Ntlm:
+      {
+        args_list << "sec=ntlm";
+        break;
+      }
+      case Smb4KCustomOptions::Ntlmi:
+      {
+        args_list << "sec=ntlmi";
+        break;
+      }
+      case Smb4KCustomOptions::Ntlmv2:
+      {
+        args_list << "sec=ntlmv2";
+        break;
+      }
+      case Smb4KCustomOptions::Ntlmv2i:
+      {
+        args_list << "sec=ntlmv2i";
+        break;
+      }
+      case Smb4KCustomOptions::Ntlmssp:
+      {
+        args_list << "sec=ntlmssp";
+        break;
+      }
+      case Smb4KCustomOptions::Ntlmsspi:
+      {
+        args_list << "sec=ntlmsspi";
+        break;
+      }
+      default:
+      {
+        // Smb4KCustomOptions::DefaultSecurityMode
+        break;
+      }
     }
-    case Smb4KSettings::EnumSecurityMode::Krb5:
+  }
+  else
+  {
+    switch ( Smb4KSettings::securityMode() )
     {
-      args_list << "sec=krb5";
-      break;
-    }
-    case Smb4KSettings::EnumSecurityMode::Krb5i:
-    {
-      args_list << "sec=krb5i";
-      break;
-    }
-    case Smb4KSettings::EnumSecurityMode::Ntlm:
-    {
-      args_list << "sec=ntlm";
-      break;
-    }
-    case Smb4KSettings::EnumSecurityMode::Ntlmi:
-    {
-      args_list << "sec=ntlmi";
-      break;
-    }
-    case Smb4KSettings::EnumSecurityMode::Ntlmv2:
-    {
-      args_list << "sec=ntlmv2";
-      break;
-    }
-    case Smb4KSettings::EnumSecurityMode::Ntlmv2i:
-    {
-      args_list << "sec=ntlmv2i";
-      break;
-    }
-    default:
-    {
-      // Smb4KSettings::EnumSecurityMode::Default,
-      break;
+      case Smb4KSettings::EnumSecurityMode::None:
+      {
+        args_list << "sec=none";
+        break;
+      }
+      case Smb4KSettings::EnumSecurityMode::Krb5:
+      {
+        args_list << "sec=krb5";
+        args_list << QString( "cruid=%1" ).arg( getuid() );
+        break;
+      }
+      case Smb4KSettings::EnumSecurityMode::Krb5i:
+      {
+        args_list << "sec=krb5i";
+        args_list << QString( "cruid=%1" ).arg( getuid() );
+        break;
+      }
+      case Smb4KSettings::EnumSecurityMode::Ntlm:
+      {
+        args_list << "sec=ntlm";
+        break;
+      }
+      case Smb4KSettings::EnumSecurityMode::Ntlmi:
+      {
+        args_list << "sec=ntlmi";
+        break;
+      }
+      case Smb4KSettings::EnumSecurityMode::Ntlmv2:
+      {
+        args_list << "sec=ntlmv2";
+        break;
+      }
+      case Smb4KSettings::EnumSecurityMode::Ntlmv2i:
+      {
+        args_list << "sec=ntlmv2i";
+        break;
+      }
+      case Smb4KSettings::EnumSecurityMode::Ntlmssp:
+      {
+        args_list << "sec=ntlmssp";
+        break;
+      }
+      case Smb4KSettings::EnumSecurityMode::Ntlmsspi:
+      {
+        args_list << "sec=ntlmsspi";
+        break;
+      }
+      default:
+      {
+        // Smb4KSettings::EnumSecurityMode::Default,
+        break;
+      }
     }
   }
 
   // Global custom options provided by the user
   if ( !Smb4KSettings::customCIFSOptions().isEmpty() )
   {
-    args_list += Smb4KSettings::customCIFSOptions().split( ",", QString::SkipEmptyParts );
+    args_list += Smb4KSettings::customCIFSOptions().split( ',', QString::SkipEmptyParts );
   }
   else
   {
@@ -543,9 +641,6 @@ bool Smb4KMountJob::createMountAction( Smb4KShare *share, Action *action )
     // Do nothing
   }
 
-  // Do not ask for a password. Use ~/.nsmbrc instead.
-//   arguments << "-N";
-
   // UID
   if ( options )
   {
@@ -673,33 +768,55 @@ bool Smb4KMountJob::createMountAction( Smb4KShare *share, Action *action )
 
   mount_command << share->canonicalPath();
 #endif
-
-  action->setName( "de.berlios.smb4k.mounthelper.mount" );
-  action->setHelperID( "de.berlios.smb4k.mounthelper" );
-  action->addArgument( "mount_command", mount_command );
+  
+  action->setName( "net.sourceforge.smb4k.mounthelper.mount" );
+  action->setHelperID( "net.sourceforge.smb4k.mounthelper" );
+  action->addArgument( "command", mount_command );
   action->addArgument( "home_dir", QDir::homePath() );
-
+  
   if ( !share->isHomesShare() )
   {
-    action->addArgument( "share_url", share->url() );
+    action->addArgument( "url", static_cast<QUrl>(share->url()) );
   }
   else
   {
-    action->addArgument( "share_url", share->homeURL() );
+    action->addArgument( "url", static_cast<QUrl>(share->homeURL()) );
   }
 
-  action->addArgument( "share_workgroup", share->workgroupName() );
-  action->addArgument( "share_comment", share->comment() );
-  action->addArgument( "share_ip", share->hostIP() );
-  action->addArgument( "share_mountpoint", share->canonicalPath() );
-
+  action->addArgument( "workgroup", share->workgroupName() );
+  action->addArgument( "comment", share->comment() );
+  action->addArgument( "ip", share->hostIP() );
+  action->addArgument( "mountpoint", share->canonicalPath() );
+ 
+  // The path to the Kerberos ticket is stored - if it exists - in the
+  // KRB5CCNAME environment variable. By default, the ticket is located
+  // at /tmp/krb5cc_[uid]. So, if the environment variable does not exist,
+  // but the cache file is there, try to use it.
+  if ( QProcessEnvironment::systemEnvironment().contains( "KRB5CCNAME" ) )
+  {
+    action->addArgument( "krb5_ticket", QProcessEnvironment::systemEnvironment().value( "KRB5CCNAME", "" ) );
+  }
+  else
+  {
+    QString ticket = QString( "/tmp/krb5cc_%1" ).arg( getuid() );
+    
+    if ( QFile::exists( ticket ) )
+    {
+      action->addArgument( "krb5_ticket", "FILE:"+ticket );
+    }
+    else
+    {
+      // Do nothing
+    }
+  }    
+ 
   return true;
 }
 
 
 bool Smb4KMountJob::doKill()
 {
-  Action( "de.berlios.smb4k.mounthelper.mount" ).stop();
+  Action( "net.sourceforge.smb4k.mounthelper.mount" ).stop();
   return KJob::doKill();
 }
 
@@ -707,17 +824,17 @@ bool Smb4KMountJob::doKill()
 void Smb4KMountJob::slotStartMount()
 {
   QList<Action> actions;
-  QMutableListIterator<Smb4KShare> it( m_shares );
+  QMutableListIterator<Smb4KShare *> it( m_shares );
 
   while ( it.hasNext() )
   {
-    Smb4KShare *share = &it.next();
+    Smb4KShare *share = it.next();
     Action mountAction;
 
     if ( createMountAction( share, &mountAction ) )
     {
-      connect( mountAction.watcher(), SIGNAL( actionPerformed( ActionReply ) ),
-               this, SLOT( slotActionFinished( ActionReply ) ) );
+      connect( mountAction.watcher(), SIGNAL(actionPerformed(ActionReply)),
+               this, SLOT(slotActionFinished(ActionReply)) );
 
       actions << mountAction;
     }
@@ -730,7 +847,7 @@ void Smb4KMountJob::slotStartMount()
   if ( !actions.isEmpty() )
   {
     emit aboutToStart( m_shares );
-    Action::executeActions( actions, NULL, "de.berlios.smb4k.mounthelper" );
+    Action::executeActions( actions, NULL, "net.sourceforge.smb4k.mounthelper" );
   }
   else
   {
@@ -748,38 +865,41 @@ void Smb4KMountJob::slotActionFinished( ActionReply reply )
 
   if ( reply.succeeded() )
   {
-    QMutableListIterator<Smb4KShare> it( m_shares );
+    QMutableListIterator<Smb4KShare *> it( m_shares );
 
     while( it.hasNext() )
     {
-      Smb4KShare *share = &it.next();
+      Smb4KShare *share = it.next();
 
       // Check if the mount process reported an error
-      QString stderr( reply.data()["stderr"].toString() );
+      QString stderr( reply.data()["stderr"].toString().trimmed() );
 
-      if ( QString::compare( share->canonicalPath(), reply.data()["share_mountpoint"].toString() ) == 0 && !stderr.isEmpty() )
+      if ( QString::compare( share->canonicalPath(), reply.data()["mountpoint"].toString() ) == 0 && !stderr.isEmpty() )
       {
 #ifndef Q_OS_FREEBSD
-        if ( stderr.contains( "mount error 13", Qt::CaseSensitive ) || stderr.contains( "mount error(13)" )
-            /* authentication error */ )
+        if ( stderr.contains( "mount error 13" ) || stderr.contains( "mount error(13)" ) /* authentication error */ )
         {
-          m_auth_errors << Smb4KShare( *share );
+          m_auth_errors << new Smb4KShare( *share );
         }
         else if ( (stderr.contains( "mount error 6" ) || stderr.contains( "mount error(6)" )) /* bad share name */ &&
                   share->shareName().contains( "_", Qt::CaseSensitive ) )
         {
           QString share_name = share->shareName();
-          share->setShareName( share_name.replace( "_", " " ) );
-          m_retries << *share;
+          share->setShareName( share_name.replace( '_', ' ' ) );
+          m_retries << new Smb4KShare( *share );
         }
         else if ( stderr.contains( "mount error 101" ) || stderr.contains( "mount error(101)" ) /* network unreachable */ )
         {
           qDebug() << "Network unreachable ..." << endl;
         }
+        else if ( stderr.contains( "Unable to find suitable address." ) )
+        {
+          // Swallow this
+        }
 #else
         if ( stderr.contains( "Authentication error" ) )
         {
-          m_auth_errors << *share;
+          m_auth_errors << new Smb4KShare( *share );
         }
 #endif
         else
@@ -811,6 +931,7 @@ void Smb4KMountJob::slotActionFinished( ActionReply reply )
 
   if ( m_processed == m_shares.size() )
   {
+    // Emit the necessary signals.
     if ( !m_auth_errors.isEmpty() )
     {
       emit authError( this );
@@ -828,22 +949,22 @@ void Smb4KMountJob::slotActionFinished( ActionReply reply )
     {
       // Do nothing
     }
-	  
+    
     // Give the operating system some time to process the mounts
     // before we invoke KMountPoint::currentMountPoints().
-    QTimer::singleShot( 100, this, SLOT( slotFinishJob() ) );
+    QTimer::singleShot( 500, this, SLOT(slotFinishJob()) );
   }
 }
 
 
 void Smb4KMountJob::slotFinishJob()
 {
-  QMutableListIterator<Smb4KShare> it( m_shares );
+  QMutableListIterator<Smb4KShare *> it( m_shares );
   Smb4KShare *share = NULL;
 
   while ( it.hasNext() )
   {
-    share = &it.next();
+    share = it.next();
 
     // Check which share has been mounted and emit the mounted() signal
     // if appropriate.
@@ -887,34 +1008,40 @@ Smb4KUnmountJob::Smb4KUnmountJob( QObject *parent ) : KJob( parent ),
 
 Smb4KUnmountJob::~Smb4KUnmountJob()
 {
+  while ( !m_shares.isEmpty() )
+  {
+    delete m_shares.takeFirst();
+  }
 }
 
 
 void Smb4KUnmountJob::start()
 {
   m_started = true;
-  QTimer::singleShot( 0, this, SLOT( slotStartUnmount() ) );
+  
+  if ( !m_synchron )
+  {
+    QTimer::singleShot( 50, this, SLOT(slotStartUnmount()) );
+  }
+  else
+  {
+    slotStartUnmount();
+  }
 }
 
 
-void Smb4KUnmountJob::synchronousStart()
-{
-  m_started = true;
-  slotStartUnmount();
-}
-
-
-void Smb4KUnmountJob::setupUnmount( Smb4KShare *share, bool force, bool silent, QWidget *parent )
+void Smb4KUnmountJob::setupUnmount( Smb4KShare *share, bool force, bool silent, bool synchron, QWidget *parent )
 {
   Q_ASSERT( share );
-  m_shares << *share;
+  m_shares << new Smb4KShare( *share );
   m_force = force;
   m_silent = silent;
+  m_synchron = synchron;
   m_parent_widget = parent;
 }
 
 
-void Smb4KUnmountJob::setupUnmount( const QList<Smb4KShare *> &shares, bool force, bool silent, QWidget* parent )
+void Smb4KUnmountJob::setupUnmount( const QList<Smb4KShare *> &shares, bool force, bool silent, bool synchron, QWidget* parent )
 {
   QListIterator<Smb4KShare *> it( shares );
 
@@ -922,16 +1049,17 @@ void Smb4KUnmountJob::setupUnmount( const QList<Smb4KShare *> &shares, bool forc
   {
     Smb4KShare *share = it.next();
     Q_ASSERT( share );
-    m_shares << *share;
+    m_shares << new Smb4KShare( *share );
   }
 
   m_force = force;
   m_silent = silent;
+  m_synchron = synchron;
   m_parent_widget = parent;
 }
 
 
-bool Smb4KUnmountJob::createUnmountAction( Smb4KShare *share, bool force, bool silent, Action *action )
+bool Smb4KUnmountJob::createUnmountAction( Smb4KShare *share, Action *action )
 {
   Q_ASSERT( share );
   Q_ASSERT( action );
@@ -960,7 +1088,7 @@ bool Smb4KUnmountJob::createUnmountAction( Smb4KShare *share, bool force, bool s
     }
   }
 
-  if ( umount.isEmpty() && !silent )
+  if ( umount.isEmpty() && !m_silent )
   {
     Smb4KNotification *notification = new Smb4KNotification();
     notification->commandNotFound( umount );
@@ -976,7 +1104,7 @@ bool Smb4KUnmountJob::createUnmountAction( Smb4KShare *share, bool force, bool s
   unmount_command << umount;
 
 #ifdef Q_OS_LINUX
-  if ( force )
+  if ( m_force )
   {
     unmount_command << "-l"; // lazy unmount
   }
@@ -988,13 +1116,13 @@ bool Smb4KUnmountJob::createUnmountAction( Smb4KShare *share, bool force, bool s
 
   unmount_command << share->canonicalPath();
 
-  action->setName( "de.berlios.smb4k.mounthelper.unmount" );
-  action->setHelperID( "de.berlios.smb4k.mounthelper" );
-  action->addArgument( "umount_command", unmount_command );
+  action->setName( "net.sourceforge.smb4k.mounthelper.unmount" );
+  action->setHelperID( "net.sourceforge.smb4k.mounthelper" );
+  action->addArgument( "command", unmount_command );
 
   // Now add everything we need.
-  action->addArgument( "share_url", share->url() );
-  action->addArgument( "share_mountpoint", share->canonicalPath() );
+  action->addArgument( "url", share->url().url() );
+  action->addArgument( "mountpoint", share->canonicalPath() );
 
   return true;
 }
@@ -1002,7 +1130,7 @@ bool Smb4KUnmountJob::createUnmountAction( Smb4KShare *share, bool force, bool s
 
 bool Smb4KUnmountJob::doKill()
 {
-  Action( "de.berlios.smb4k.mounthelper.unmount" ).stop();
+  Action( "net.sourceforge.smb4k.mounthelper.unmount" ).stop();
   return KJob::doKill();
 }
 
@@ -1010,17 +1138,17 @@ bool Smb4KUnmountJob::doKill()
 void Smb4KUnmountJob::slotStartUnmount()
 {
   QList<Action> actions;
-  QMutableListIterator<Smb4KShare> it( m_shares );
+  QMutableListIterator<Smb4KShare *> it( m_shares );
 
   while ( it.hasNext() )
   {
-    Smb4KShare *share = &it.next();
+    Smb4KShare *share = it.next();
     Action unmountAction;
 
-    if ( createUnmountAction( share, m_force, m_silent, &unmountAction ) )
+    if ( createUnmountAction( share, &unmountAction ) )
     {
-      connect( unmountAction.watcher(), SIGNAL( actionPerformed( ActionReply ) ),
-               this, SLOT( slotActionFinished( ActionReply ) ) );
+      connect( unmountAction.watcher(), SIGNAL(actionPerformed(ActionReply)),
+               this, SLOT(slotActionFinished(ActionReply)) );
 
       actions << unmountAction;
     }
@@ -1033,7 +1161,7 @@ void Smb4KUnmountJob::slotStartUnmount()
   if ( !actions.isEmpty() )
   {
     emit aboutToStart( m_shares );
-    Action::executeActions( actions, NULL, "de.berlios.smb4k.mounthelper" );
+    Action::executeActions( actions, NULL, "net.sourceforge.smb4k.mounthelper" );
   }
   else
   {
@@ -1054,16 +1182,17 @@ void Smb4KUnmountJob::slotActionFinished( ActionReply reply )
     // return the right url, it seems that this is not the
     // case with bulk operations. So, we just check which of
     // the shares has been mounted and emit the mounted signal.
-    QMutableListIterator<Smb4KShare> it( m_shares );
+    QMutableListIterator<Smb4KShare *> it( m_shares );
 
     while( it.hasNext() )
     {
-      Smb4KShare *share = &it.next();
+      Smb4KShare *share = it.next();
 
       // Check if the unmount process reported an error
-      QString stderr( reply.data()["stderr"].toString() );
+      QString stderr( reply.data()["stderr"].toString().trimmed() );
 
-      if ( QString::compare( share->canonicalPath(), reply.data()["share_mountpoint"].toString() ) == 0 && !stderr.isEmpty() )
+      if ( QString::compare( share->canonicalPath(), reply.data()["mountpoint"].toString() ) == 0 && 
+           !stderr.isEmpty() && !m_silent )
       {
         Smb4KNotification *notification = new Smb4KNotification();
         notification->unmountingFailed( share, stderr );
@@ -1078,15 +1207,22 @@ void Smb4KUnmountJob::slotActionFinished( ActionReply reply )
   else
   {
     // The auth action failed. Report this.
-    Smb4KNotification *notification = new Smb4KNotification();
-
-    if ( reply.type() == ActionReply::KAuthError )
+    if ( !m_silent )
     {
-      notification->actionFailed( reply.errorCode() );
+      Smb4KNotification *notification = new Smb4KNotification();
+
+      if ( reply.type() == ActionReply::KAuthError )
+      {
+        notification->actionFailed( reply.errorCode() );
+      }
+      else
+      {
+        notification->actionFailed();
+      }
     }
     else
     {
-      notification->actionFailed();
+      // Do nothing
     }
   }
 
@@ -1096,19 +1232,19 @@ void Smb4KUnmountJob::slotActionFinished( ActionReply reply )
     // before we invoke KMountPoint::currentMountPoints(). It seems
     // that we need at least 500 ms, so that even slow systems have
     // the opportunity to unregister the mounts.
-    QTimer::singleShot( 500, this, SLOT( slotFinishJob() ) );
+    QTimer::singleShot( 500, this, SLOT(slotFinishJob()) );
   }
 }
 
 
 void Smb4KUnmountJob::slotFinishJob()
 {
-  QMutableListIterator<Smb4KShare> it( m_shares );
+  QMutableListIterator<Smb4KShare *> it( m_shares );
   Smb4KShare *share = NULL;
 
   while ( it.hasNext() )
   {
-    share = &it.next();
+    share = it.next();
 
     // Check if the share has been unmounted and emit the unmounted()
     // signal if appropriate.
@@ -1163,8 +1299,8 @@ Smb4KMountDialog::Smb4KMountDialog( Smb4KShare *share, QWidget *parent )
 
   setupView();
 
-  connect( this, SIGNAL( okClicked() ), SLOT( slotOkClicked() ) );
-  connect( this, SIGNAL( cancelClicked() ), SLOT( slotCancelClicked() ) );
+  connect( this, SIGNAL(okClicked()), SLOT(slotOkClicked()) );
+  connect( this, SIGNAL(cancelClicked()), SLOT(slotCancelClicked()) );
 
   setMinimumWidth( sizeHint().width() > 350 ? sizeHint().width() : 350 );
 
@@ -1266,17 +1402,17 @@ void Smb4KMountDialog::setupView()
   slotChangeInputValue( m_share_input->text() );
 
   // Connections
-  connect( m_share_input,     SIGNAL( textChanged ( const QString & ) ) ,
-           this,              SLOT( slotChangeInputValue( const QString & ) ) );
+  connect( m_share_input,     SIGNAL(textChanged(QString)) ,
+           this,              SLOT(slotChangeInputValue(QString)) );
 
-  connect( m_share_input,     SIGNAL( editingFinished() ),
-           this,              SLOT( slotShareNameEntered() ) );
+  connect( m_share_input,     SIGNAL(editingFinished()),
+           this,              SLOT(slotShareNameEntered()) );
 
-  connect( m_ip_input,        SIGNAL( editingFinished() ),
-           this,              SLOT( slotIPEntered() ) );
+  connect( m_ip_input,        SIGNAL(editingFinished()),
+           this,              SLOT(slotIPEntered()) );
 
-  connect( m_workgroup_input, SIGNAL( editingFinished() ),
-           this,              SLOT( slotWorkgroupEntered() ) );
+  connect( m_workgroup_input, SIGNAL(editingFinished()),
+           this,              SLOT(slotWorkgroupEntered()) );
 }
 
 
@@ -1294,24 +1430,23 @@ void Smb4KMountDialog::slotOkClicked()
 {
   if ( !m_share_input->text().trimmed().isEmpty() )
   {
-    QUrl url;
-    
+    KUrl url;
+
     // Take care of Windows-like UNC addresses:
-    if ( m_share_input->text().trimmed().startsWith( "\\" ) )
+    if ( m_share_input->text().trimmed().startsWith( QLatin1String( "\\" ) ) )
     {
       QString unc = m_share_input->text();
       unc.replace( "\\", "/" );
-      url = QUrl( unc );
+      url.setUrl( unc, KUrl::TolerantMode );
     }
     else
     {
-      url = QUrl( m_share_input->text().trimmed() );
+      url.setUrl( m_share_input->text().trimmed(), KUrl::TolerantMode );
     }
-    
-    url.setScheme( "smb" );
 
-    if ( url.isValid() && !url.host().isEmpty() /* no invalid host name */ &&
-         url.path().length() > 1 /* share name length */ && !url.path().endsWith( "/" ) )
+    url.setProtocol( "smb" );
+
+    if ( url.isValid() && url.hasHost() && url.hasPath() && !url.path().endsWith( '/' ) )
     {
       m_share->setURL( url );
       m_share->setWorkgroupName( m_workgroup_input->text().trimmed() );
@@ -1346,8 +1481,8 @@ void Smb4KMountDialog::slotCancelClicked()
 void Smb4KMountDialog::slotShareNameEntered()
 {
   KCompletion *completion = m_share_input->completionObject();
-  QUrl url( m_share_input->userText() );
-  url.setScheme( "smb" );
+  KUrl url( m_share_input->userText() );
+  url.setProtocol( "smb" );
 
   if ( url.isValid() && !url.isEmpty() )
   {
@@ -1390,67 +1525,4 @@ void Smb4KMountDialog::slotWorkgroupEntered()
 }
 
 
-Smb4KMounterPrivate::Smb4KMounterPrivate()
-: m_quit( false ), m_hardware( false ), m_pending_unmounts( 0 ), m_initial_unmounts( 0 ),
-  m_pending_mounts( 0 ), m_initial_mounts( 0 )
-{
-}
-
-
-Smb4KMounterPrivate::~Smb4KMounterPrivate()
-{
-}
-
-
-void Smb4KMounterPrivate::setAboutToQuit()
-{
-  m_quit = true;
-}
-
-
-void Smb4KMounterPrivate::setHardwareReason( bool hardware )
-{
-  m_hardware = hardware;
-}
-
-
-void Smb4KMounterPrivate::addUnmount()
-{
-  m_initial_unmounts++;
-  m_pending_unmounts++;
-}
-
-
-void Smb4KMounterPrivate::removeUnmount()
-{
-  m_pending_unmounts--;
-}
-
-
-void Smb4KMounterPrivate::clearUnmounts()
-{
-  m_initial_unmounts = 0;
-  m_pending_unmounts = 0;
-}
-
-
-void Smb4KMounterPrivate::addMount()
-{
-  m_initial_mounts++;
-  m_pending_mounts++;
-}
-
-
-void Smb4KMounterPrivate::removeMount()
-{
-  m_pending_mounts--;
-}
-
-
-void Smb4KMounterPrivate::clearMounts()
-{
-  m_initial_mounts = 0;
-  m_pending_mounts = 0;
-}
-
 #include "smb4kmounter_p.moc"
