@@ -2,7 +2,7 @@
     smb4kpreviewer  -  This class queries a remote share for a preview
                              -------------------
     begin                : Sa Mär 05 2011
-    copyright            : (C) 2011 by Alexander Reinholdt
+    copyright            : (C) 2011-2012 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -23,30 +23,47 @@
  *   MA 02110-1335, USA                                                    *
  ***************************************************************************/
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+// application specific includes
+#include "smb4kpreviewer.h"
+#include "smb4kpreviewer_p.h"
+#include "smb4kwalletmanager.h"
+#include "smb4kshare.h"
+#include "smb4kauthinfo.h"
+#include "smb4khomesshareshandler.h"
+#include "smb4kglobal.h"
+
 // Qt includes
-#include <QTimer>
-#include <QDebug>
-#include <QCoreApplication>
+#include <QtCore/QTimer>
+#include <QtCore/QDebug>
+#include <QtCore/QCoreApplication>
 
 // KDE includes
 #include <kglobal.h>
 
-// application specific includes
-#include <smb4kpreviewer.h>
-#include <smb4kpreviewer_p.h>
-#include <smb4kwalletmanager.h>
-#include <smb4kshare.h>
-#include <smb4kauthinfo.h>
-#include <smb4khomesshareshandler.h>
+using namespace Smb4KGlobal;
 
-K_GLOBAL_STATIC( Smb4KPreviewerPrivate, p );
+K_GLOBAL_STATIC( Smb4KPreviewerStatic, p );
 
 
-Smb4KPreviewer::Smb4KPreviewer() : KCompositeJob( 0 )
+Smb4KPreviewer::Smb4KPreviewer( QObject *parent )
+: KCompositeJob( parent ), d( new Smb4KPreviewerPrivate )
 {
   setAutoDelete( false );
+
+  if ( !coreIsInitialized() )
+  {
+    setDefaultSettings();
+  }
+  else
+  {
+    // Do nothing
+  }
   
-  connect( QCoreApplication::instance(), SIGNAL( aboutToQuit() ), SLOT( slotAboutToQuit() ) );
+  connect( QCoreApplication::instance(), SIGNAL(aboutToQuit()), SLOT(slotAboutToQuit()) );
 }
 
 
@@ -93,11 +110,11 @@ void Smb4KPreviewer::preview( Smb4KShare *share, QWidget *parent )
   // for this share and reuse it, if appropriate.
   Smb4KPreviewDialog *dlg = NULL;
   
-  for ( int i = 0; i < m_dialogs.size(); ++i )
+  for ( int i = 0; i < d->dialogs.size(); ++i )
   {
-    if ( share == m_dialogs.at( i )->share() )
+    if ( share == d->dialogs.at( i )->share() )
     {
-      dlg = m_dialogs.at( i );
+      dlg = d->dialogs.at( i );
     }
     else
     {
@@ -109,17 +126,17 @@ void Smb4KPreviewer::preview( Smb4KShare *share, QWidget *parent )
   {
     // Create the preview dialog..
     dlg = new Smb4KPreviewDialog( share, parent );
-    connect( dlg,  SIGNAL( aboutToClose( Smb4KPreviewDialog * ) ),
-             this, SLOT( slotDialogClosed( Smb4KPreviewDialog * ) ) );
-    connect( dlg,  SIGNAL( requestPreview( Smb4KShare *, const QUrl &, QWidget * ) ),
-             this, SLOT( slotAcquirePreview( Smb4KShare *, const QUrl &, QWidget * ) ) );
-    connect( this, SIGNAL( aboutToStart( Smb4KShare *, const QUrl & ) ),
-             dlg,  SLOT( slotAboutToStart( Smb4KShare *, const QUrl & ) ) );
-    connect( this, SIGNAL( finished( Smb4KShare *, const QUrl & ) ),
-             dlg,  SLOT( slotFinished( Smb4KShare *, const QUrl & ) ) );
-    connect( dlg,  SIGNAL( abortPreview( Smb4KShare * ) ),
-             this, SLOT( slotAbortPreview( Smb4KShare* ) ) );
-    m_dialogs.append( dlg );
+    connect( dlg,  SIGNAL(aboutToClose(Smb4KPreviewDialog*)),
+             this, SLOT(slotDialogClosed(Smb4KPreviewDialog*)) );
+    connect( dlg,  SIGNAL(requestPreview(Smb4KShare*,KUrl,QWidget*)),
+             this, SLOT(slotAcquirePreview(Smb4KShare*,KUrl,QWidget*)) );
+    connect( this, SIGNAL(aboutToStart(Smb4KShare*,KUrl)),
+             dlg,  SLOT(slotAboutToStart(Smb4KShare*,KUrl)) );
+    connect( this, SIGNAL(finished(Smb4KShare*,KUrl)),
+             dlg,  SLOT(slotFinished(Smb4KShare*,KUrl)) );
+    connect( dlg,  SIGNAL(abortPreview(Smb4KShare*)),
+             this, SLOT(slotAbortPreview(Smb4KShare*)) );
+    d->dialogs.append( dlg );
   }
   else
   {
@@ -139,7 +156,7 @@ void Smb4KPreviewer::preview( Smb4KShare *share, QWidget *parent )
 
 bool Smb4KPreviewer::isRunning()
 {
-  return !subjobs().isEmpty();
+  return hasSubjobs();
 }
 
 
@@ -213,7 +230,7 @@ void Smb4KPreviewer::abort( Smb4KShare *share )
 
 void Smb4KPreviewer::start()
 {
-  QTimer::singleShot( 0, this, SLOT( slotStartJobs() ) );
+  QTimer::singleShot( 0, this, SLOT(slotStartJobs()) );
 }
 
 
@@ -254,8 +271,8 @@ void Smb4KPreviewer::slotDialogClosed( Smb4KPreviewDialog *dialog )
     // Find the dialog in the list and take it from the list.
     // It will automatically be deleted on close, so there is
     // no need to delete the dialog here.
-    int i = m_dialogs.indexOf( dialog );
-    m_dialogs.takeAt( i );
+    int i = d->dialogs.indexOf( dialog );
+    d->dialogs.takeAt( i );
   }
   else
   {
@@ -264,7 +281,7 @@ void Smb4KPreviewer::slotDialogClosed( Smb4KPreviewDialog *dialog )
 }
 
 
-void Smb4KPreviewer::slotAcquirePreview( Smb4KShare *share, const QUrl &url, QWidget *parent )
+void Smb4KPreviewer::slotAcquirePreview( Smb4KShare *share, const KUrl &url, QWidget *parent )
 {
   // Get the authentication information
   Smb4KWalletManager::self()->readAuthInfo( share );
@@ -282,24 +299,24 @@ void Smb4KPreviewer::slotAcquirePreview( Smb4KShare *share, const QUrl &url, QWi
   }
   job->setupPreview( share, url, parent );
 
-  connect( job,  SIGNAL( result( KJob * ) ),
-           this, SLOT( slotJobFinished( KJob * ) ) );
-  connect( job,  SIGNAL( authError( Smb4KPreviewJob * ) ),
-           this, SLOT( slotAuthError( Smb4KPreviewJob * ) ) );
-  connect( job,  SIGNAL( aboutToStart( Smb4KShare *, const QUrl & ) ),
-           this, SIGNAL( aboutToStart( Smb4KShare *, const QUrl & ) ) );
-  connect( job,  SIGNAL( finished( Smb4KShare *, const QUrl & ) ),
-           this, SIGNAL( finished( Smb4KShare *, const QUrl & ) ) );
+  connect( job,  SIGNAL(result(KJob*)),
+           this, SLOT(slotJobFinished(KJob*)) );
+  connect( job,  SIGNAL(authError(Smb4KPreviewJob*)),
+           this, SLOT(slotAuthError(Smb4KPreviewJob*)) );
+  connect( job,  SIGNAL(aboutToStart(Smb4KShare*,KUrl)),
+           this, SIGNAL(aboutToStart(Smb4KShare*,KUrl)) );
+  connect( job,  SIGNAL(finished(Smb4KShare*,KUrl)),
+           this, SIGNAL(finished(Smb4KShare*,KUrl)) );
 
   // Get the preview dialog, so that the result of the query
   // can be sent.
   Smb4KPreviewDialog *dlg = NULL;
 
-  for ( int i = 0; i < m_dialogs.size(); ++i )
+  for ( int i = 0; i < d->dialogs.size(); ++i )
   {
-    if ( m_dialogs.at( i ) && m_dialogs.at( i )->share() == share )
+    if ( d->dialogs.at( i ) && d->dialogs.at( i )->share() == share )
     {
-      dlg = m_dialogs[i];
+      dlg = d->dialogs[i];
       break;
     }
     else
@@ -310,8 +327,8 @@ void Smb4KPreviewer::slotAcquirePreview( Smb4KShare *share, const QUrl &url, QWi
   
   if ( dlg )
   {
-    connect( job, SIGNAL( preview( const QUrl &, const QList<Item> & ) ),
-             dlg, SLOT( slotDisplayPreview( const QUrl &, const QList<Item> & ) ) );
+    connect( job, SIGNAL(preview(KUrl,QList<Item>)),
+             dlg, SLOT(slotDisplayPreview(KUrl,QList<Item>)) );
   }
   else
   {
