@@ -2,7 +2,7 @@
     smb4kpreviewer_p  -  Private helper classes for Smb4KPreviewer class.
                              -------------------
     begin                : So Dez 21 2008
-    copyright            : (C) 2008-2011 by Alexander Reinholdt
+    copyright            : (C) 2008-2013 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -19,14 +19,29 @@
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
- *   Free Software Foundation, 51 Franklin Street, Suite 500, Boston,      *
+ *   Free Software Foundation, Inc., 51 Franklin Street, Suite 500, Boston,*
  *   MA 02110-1335, USA                                                    *
  ***************************************************************************/
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+// application specific includes
+#include "smb4kpreviewer_p.h"
+#include "smb4knotification.h"
+#include "smb4khost.h"
+#include "smb4kshare.h"
+#include "smb4kglobal.h"
+#include "smb4ksettings.h"
+#include "smb4kcustomoptionsmanager.h"
+#include "smb4kcustomoptions.h"
+
 // Qt includes
-#include <QTimer>
-#include <QGridLayout>
-#include <QDateTime>
+#include <QtCore/QTimer>
+#include <QtCore/QDateTime>
+#include <QtCore/QLatin1String>
+#include <QtGui/QGridLayout>
 
 // KDE includes
 #include <kdebug.h>
@@ -37,16 +52,6 @@
 #include <kstatusbar.h>
 #include <kshell.h>
 
-// application specific includes
-#include <smb4kpreviewer_p.h>
-#include <smb4knotification.h>
-#include <smb4khost.h>
-#include <smb4kshare.h>
-#include <smb4kglobal.h>
-#include <smb4ksettings.h>
-#include <smb4kcustomoptionsmanager.h>
-#include <smb4kcustomoptions.h>
-
 using namespace Smb4KGlobal;
 
 
@@ -64,11 +69,11 @@ Smb4KPreviewJob::~Smb4KPreviewJob()
 void Smb4KPreviewJob::start()
 {
   m_started = true;
-  QTimer::singleShot( 0, this, SLOT( slotStartPreview() ) );
+  QTimer::singleShot( 0, this, SLOT(slotStartPreview()) );
 }
 
 
-void Smb4KPreviewJob::setupPreview( Smb4KShare *share, const QUrl &url, QWidget *parent )
+void Smb4KPreviewJob::setupPreview( Smb4KShare *share, const KUrl &url, QWidget *parent )
 {
   Q_ASSERT( share );
   m_share = share;
@@ -113,17 +118,17 @@ void Smb4KPreviewJob::slotStartPreview()
   emit aboutToStart( m_share, m_url );
 
   // Get the path that has to be listed.
-  QString path = m_url.toString( QUrl::RemoveScheme|QUrl::RemoveUserInfo|QUrl::RemovePort );
+  QString path = m_url.path();
   
   if ( !m_share->isHomesShare() )
   {
-    path.remove( m_share->unc(), Qt::CaseInsensitive );
+    path.remove( m_share->shareName(), Qt::CaseInsensitive ); 
   }
   else
   {
-    path.remove( m_share->homeUNC(), Qt::CaseInsensitive );
+    path.remove( m_share->login(), Qt::CaseInsensitive );
   }
-
+  
   // Compile the command line arguments
   QStringList arguments;
   
@@ -337,9 +342,9 @@ void Smb4KPreviewJob::slotStartPreview()
   m_proc->setEnv( "PASSWD", m_share->password(), true );
   m_proc->setShellCommand( arguments.join( " " ) );
 
-  connect( m_proc, SIGNAL( readyReadStandardOutput() ), SLOT( slotReadStandardOutput() ) );
-  connect( m_proc, SIGNAL( readyReadStandardError() ),  SLOT( slotReadStandardError() ) );
-  connect( m_proc, SIGNAL( finished( int, QProcess::ExitStatus ) ), SLOT( slotProcessFinished( int, QProcess::ExitStatus ) ) );
+  connect( m_proc, SIGNAL(readyReadStandardOutput()), SLOT(slotReadStandardOutput()) );
+  connect( m_proc, SIGNAL(readyReadStandardError()),  SLOT(slotReadStandardError()) );
+  connect( m_proc, SIGNAL(finished(int,QProcess::ExitStatus)), SLOT(slotProcessFinished(int,QProcess::ExitStatus)) );
 
   m_proc->start();
 }
@@ -347,7 +352,7 @@ void Smb4KPreviewJob::slotStartPreview()
 
 void Smb4KPreviewJob::slotReadStandardOutput()
 {
-  QStringList list = QString::fromUtf8( m_proc->readAllStandardOutput(), -1 ).split( "\n", QString::SkipEmptyParts );
+  QStringList list = QString::fromUtf8( m_proc->readAllStandardOutput(), -1 ).split( '\n', QString::SkipEmptyParts );
   QList<Item> items;
 
   foreach ( const QString &line, list )
@@ -398,7 +403,7 @@ void Smb4KPreviewJob::slotReadStandardOutput()
 
         if ( is_dir )
         {
-          if ( name.startsWith( "." ) &&
+          if ( name.startsWith( '.' ) &&
               (QString::compare( name, "." ) != 0 && QString::compare( name, ".." ) != 0) )
           {
             item.first = HiddenDirectoryItem;
@@ -410,7 +415,7 @@ void Smb4KPreviewJob::slotReadStandardOutput()
         }
         else
         {
-          if ( name.startsWith( "." ) )
+          if ( name.startsWith( '.' ) )
           {
             item.first = HiddenFileItem;
           }
@@ -459,7 +464,7 @@ void Smb4KPreviewJob::slotReadStandardError()
     {
       it.remove();
     }
-    else if ( line.trimmed().startsWith( "Ignoring unknown parameter" ) )
+    else if ( line.trimmed().startsWith( QLatin1String( "Ignoring unknown parameter" ) ) )
     {
       it.remove();
     }
@@ -561,15 +566,15 @@ Smb4KPreviewDialog::Smb4KPreviewDialog( Smb4KShare *share, QWidget *parent )
 
   setupView();
 
-  connect( this,                   SIGNAL( closeClicked() ),
-           this,                   SLOT( slotCloseClicked() ) );
+  connect( this,                   SIGNAL(closeClicked()),
+           this,                   SLOT(slotCloseClicked()) );
 
   setMinimumWidth( sizeHint().width() > 350 ? sizeHint().width() : 350 );
 
   KConfigGroup group( Smb4KSettings::self()->config(), "PreviewDialog" );
   restoreDialogSize( group );
 
-  QTimer::singleShot( 0, this, SLOT( slotRequestPreview() ) );
+  QTimer::singleShot( 0, this, SLOT(slotRequestPreview()) );
 }
 
 
@@ -630,17 +635,17 @@ void Smb4KPreviewDialog::setupView()
   layout->addWidget( m_view, 0, 0, 0 );
   layout->addWidget( toolbar, 1, 0, 0 );
 
-  connect( toolbar, SIGNAL( actionTriggered( QAction * ) ),
-           this,      SLOT( slotActionTriggered( QAction * ) ) );
+  connect( toolbar, SIGNAL(actionTriggered(QAction*)),
+           this,      SLOT(slotActionTriggered(QAction*)) );
 
-  connect( m_combo,   SIGNAL( activated( const QString & ) ),
-           this,      SLOT( slotItemActivated( const QString & ) ) );
+  connect( m_combo,   SIGNAL(activated(QString)),
+           this,      SLOT(slotItemActivated(QString)) );
 
-  connect( m_view,    SIGNAL( executed( QListWidgetItem * ) ),
-           this,      SLOT( slotItemExecuted( QListWidgetItem * ) ) );
+  connect( m_view,    SIGNAL(executed(QListWidgetItem*)),
+           this,      SLOT(slotItemExecuted(QListWidgetItem*)) );
 
-  connect( KGlobalSettings::self(), SIGNAL( iconChanged( int ) ),
-           this,                    SLOT( slotIconSizeChanged( int ) ) );
+  connect( KGlobalSettings::self(), SIGNAL(iconChanged(int)),
+           this,                    SLOT(slotIconSizeChanged(int)) );
 }
 
 
@@ -789,14 +794,18 @@ void Smb4KPreviewDialog::slotActionTriggered( QAction *action )
     }
     else if ( kaction == m_up )
     {
-      if ( QString::compare( m_share->unc(), m_url.toString( QUrl::RemoveScheme|QUrl::RemoveUserInfo|QUrl::RemovePort ), Qt::CaseInsensitive ) != 0 )
+      QString test = QString( "//%1/%2" ).arg( m_url.host() ).arg( m_url.path(KUrl::RemoveTrailingSlash) );
+      
+      if ( QString::compare( m_share->unc(), test, Qt::CaseInsensitive ) != 0 )
       {
         // Clear the history
         m_history.clear();
 
         // Adjust the path
+        // FIXME: Use KUrl::upUrl() here when we have found to adjust 
+        // the path
         QString path = m_url.path();
-        m_url.setPath( path.section( "/", 0, -2 ) );
+        m_url.setPath( path.section( '/', 0, -2 ) );
 
         // Request the preview
         slotRequestPreview();
@@ -820,9 +829,14 @@ void Smb4KPreviewDialog::slotActionTriggered( QAction *action )
 
 void Smb4KPreviewDialog::slotRequestPreview()
 {
-  // Set the current URL in the combo box
-  QString current = m_url.toString( QUrl::RemoveScheme|QUrl::RemoveUserInfo|QUrl::RemovePort ).replace( m_url.host(), m_share->hostName() );
-
+  // Set the current item
+  KUrl u = m_url;
+  u.setProtocol( QString() );
+  u.setUserInfo( QString() );
+  u.setPort( -1 );
+  
+  QString current = u.url().replace( u.host(), u.host().toUpper() );
+    
   // Set the history
   QStringList history;
 
@@ -849,7 +863,7 @@ void Smb4KPreviewDialog::slotRequestPreview()
 }
 
 
-void Smb4KPreviewDialog::slotDisplayPreview( const QUrl &url, const QList<Item> &contents )
+void Smb4KPreviewDialog::slotDisplayPreview( const KUrl &url, const QList<Item> &contents )
 {
   if ( m_url != url )
   {
@@ -934,14 +948,13 @@ void Smb4KPreviewDialog::slotDisplayPreview( const QUrl &url, const QList<Item>
   m_forward->setEnabled( enable_forward );
 
   // Enable/disable the up action.
-  bool enable_up = (QString::compare( m_share->unc(),
-                                      m_url.toString( QUrl::RemoveScheme|QUrl::RemoveUserInfo|QUrl::RemovePort ),
-                                      Qt::CaseInsensitive ) != 0);
+  QString test = QString( "//%1/%2" ).arg( m_url.host() ).arg( m_url.path(KUrl::RemoveTrailingSlash) );
+  bool enable_up = (QString::compare( m_share->unc(), test, Qt::CaseInsensitive ) != 0);
   m_up->setEnabled( enable_up );
 }
 
 
-void Smb4KPreviewDialog::slotAboutToStart( Smb4KShare *share, const QUrl &url )
+void Smb4KPreviewDialog::slotAboutToStart( Smb4KShare *share, const KUrl &url )
 {
   if ( share == m_share && url == m_url )
   {
@@ -955,7 +968,7 @@ void Smb4KPreviewDialog::slotAboutToStart( Smb4KShare *share, const QUrl &url )
 }
 
 
-void Smb4KPreviewDialog::slotFinished( Smb4KShare *share, const QUrl &url )
+void Smb4KPreviewDialog::slotFinished( Smb4KShare *share, const KUrl &url )
 {
   if ( share == m_share && url == m_url )
   {
@@ -1012,7 +1025,11 @@ void Smb4KPreviewDialog::slotItemActivated( const QString &item )
     // Clear the history
     m_history.clear();
 
-    m_url.setPath( QUrl( item ).path() );
+    KUrl u;
+    u.setUrl( item, KUrl::TolerantMode );
+    u.setProtocol( "smb" );
+    
+    m_url.setPath( u.path() );
     slotRequestPreview();
   }
   else
@@ -1048,14 +1065,5 @@ void Smb4KPreviewDialog::slotIconSizeChanged( int group )
 }
 
 
-Smb4KPreviewerPrivate::Smb4KPreviewerPrivate()
-{
-}
-
-
-Smb4KPreviewerPrivate::~Smb4KPreviewerPrivate()
-{
-}
-
 #include "smb4kpreviewer_p.moc"
 
