@@ -2,7 +2,7 @@
     smb4knetworkbrowser  -  The network browser widget of Smb4K.
                              -------------------
     begin                : Mo Jan 8 2007
-    copyright            : (C) 2007-2011 by Alexander Reinholdt
+    copyright            : (C) 2007-2013 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -19,17 +19,13 @@
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
- *   Free Software Foundation, 51 Franklin Street, Suite 500, Boston,      *
+ *   Free Software Foundation, Inc., 51 Franklin Street, Suite 500, Boston,*
  *   MA 02110-1335, USA                                                    *
  ***************************************************************************/
 
 #ifndef SMB4KNETWORKBROWSER_H
 #define SMB4KNETWORKBROWSER_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 // Qt includes
 #include <QTreeWidget>
 
@@ -47,7 +43,7 @@ class Smb4KNetworkBrowser : public QTreeWidget
      *
      * @param parent        The parent widget
      */
-    Smb4KNetworkBrowser( QWidget *parent = 0 );
+    explicit Smb4KNetworkBrowser( QWidget *parent = 0 );
 
     /**
      * The destructor
@@ -62,15 +58,6 @@ class Smb4KNetworkBrowser : public QTreeWidget
                   IP = 2,
                   Comment = 3 };
 
-    /**
-     * Returns a pointer to the current tool tip or NULL, if there
-     * is no tool tip at the moment. Please note, that the tool tip
-     * is generated 2 sec before it is shown.
-     *
-     * @returns a pointer to the current tool tip.
-     */
-    Smb4KToolTip *tooltip() const { return m_tooltip; }
-
     /**
      * This function returns TRUE if the mouse is inside the network
      * browser widget and FALSE otherwise.
@@ -89,6 +76,20 @@ class Smb4KNetworkBrowser : public QTreeWidget
      */
     void itemExecuted( QTreeWidgetItem *item,
                        int column );
+    
+    /**
+     * This signal is emitted when a tool tip is about to be shown.
+     * 
+     * @param item          The network browser item
+     */
+    void aboutToShowToolTip( Smb4KNetworkBrowserItem *item );
+    
+    /**
+     * This signal is emitted when a tool tip is about to be hidden.
+     * 
+     * @param item          The network browser item
+     */
+    void aboutToHideToolTip( Smb4KNetworkBrowserItem *item );
 
   protected:
     /**
@@ -186,10 +187,10 @@ class Smb4KNetworkBrowser : public QTreeWidget
 
   private:
     /**
-     * The tool tip for the network browser
+     * The item for that a tool tip is shown
      */
-    Smb4KToolTip *m_tooltip;
-
+    Smb4KNetworkBrowserItem *m_tooltip_item;
+    
     /**
      * Mouse inside the widget?
      */
